## This script
# - Opens the .csv file containing the acoustic measures made on the identification stimuli
# - Draws plot of the acoustic cues for various parameters

# Written by Marc Brunelle in 2019

## Load libraries and set working directory

library(ggplot2)
library(ggpubr)
detach("package:Hmisc", unload=TRUE) #In case it is loaded

theme_set(theme_bw())

setwd("C:\\Users\\Owner\\Desktop\\Chru_paper\\Chru_paper_April_2020\\Data\\Acoustic_measures_synthesized_data")

## load PraatSauce data for mta
ps <- read.csv('Chrusynth_mta.txt')

# Fill names of columns and turn characters into factors
names(ps)[3] <- 'F1lab'
names(ps)[4] <- 'f0lab'
names(ps)[5] <- 'VQlab'
ps$F1lab <- factor(ps$F1lab)
ps$f0lab <- factor(ps$f0lab)
ps$VQlab <- factor(ps$VQlab)


## Create charts

# Remove first two t points to remove effect of onset on measures (10 ms, equivalent to the 12ms in other charts))
ps <- droplevels(subset(ps, t > 2))  
ps$t_ms <- ps$t * 5

f0agg <- ddply(ps, c("f0lab", "t_ms"), summarize, N = sum(!is.na(f0)), meanf0 = mean(f0, na.rm=TRUE), sd = sd(f0, na.rm=TRUE), se = sd / sqrt(N))
F1agg <- ddply(ps, c("F1lab", "t_ms"), summarize, N = sum(!is.na(F1)), meanF1 = mean(F1, na.rm=TRUE), sd = sd(F1, na.rm=TRUE), se = sd / sqrt(N))
H1H2agg <- ddply(ps, c("VQlab", "t_ms"), summarize, N = sum(!is.na(H1H2c)), meanH1H2c = mean(H1H2c, na.rm=TRUE), sd = sd(H1H2c, na.rm=TRUE), se = sd / sqrt(N))
CPPagg <- ddply(ps, c("VQlab", "t_ms"), summarize, N = sum(!is.na(CPP)), meanCPP = mean(CPP, na.rm=TRUE), sd = sd(CPP, na.rm=TRUE), se = sd / sqrt(N))

##charts for paper
F1a <- ggplot(F1agg, aes(x=t_ms, y=meanF1, group = F1lab)) + geom_line(aes(color=F1lab, linetype=F1lab), size =1) + geom_ribbon(aes(ymin=meanF1-sd, ymax=meanF1+sd, fill=F1lab), alpha=0.2) + labs(x=element_blank(), y = "F1 (Hz)") + scale_colour_discrete(name = "F1") + scale_fill_discrete(name = "F1") + scale_linetype_discrete(name = "F1") + theme(text = element_text(size = 10), legend.position="top", legend.key.size=unit(0.5, "line"), legend.text = element_text(size=8), legend.title = element_text(size=8)) + scale_x_continuous(breaks=seq(0, 300, 100))
f0a <- ggplot(f0agg, aes(x=t_ms, y=meanf0, group = f0lab)) + geom_line(aes(color=f0lab, linetype=f0lab), size =1) + geom_ribbon(aes(ymin=meanf0-sd, ymax=meanf0+sd, fill=f0lab), alpha=0.2) + labs(x =element_blank(), y = "f0 (Hz)") + scale_colour_discrete(name = "f0") + scale_fill_discrete(name = "f0") + scale_linetype_discrete(name = "f0") + theme(text = element_text(size = 10), legend.position="top", legend.key.size=unit(0.5, "line"), legend.text = element_text(size=8), legend.title = element_text(size=8)) + scale_x_continuous(breaks=seq(0, 300, 100))
H1H2a <- ggplot(H1H2agg, aes(x=t_ms, y=meanH1H2c, group = VQlab)) + geom_line(aes(color=VQlab, linetype=VQlab), size =1) + geom_ribbon(aes(ymin=meanH1H2c-sd, ymax=meanH1H2c+sd, fill=VQlab), alpha=0.2) + labs(x=element_blank(), y = "H1*-H2* (dB)") + scale_colour_discrete(name = "OQ", labels=c("0.4", "0.5", "0.6")) + scale_fill_discrete(name = "OQ", labels=c("0.4", "0.5", "0.6")) + scale_linetype_discrete(name = "OQ",, labels=c("0.4", "0.5", "0.6")) + theme(text = element_text(size = 10), legend.position="top", legend.key.size=unit(0.5, "line"), legend.text = element_text(size=8), legend.title = element_text(size=8)) + scale_x_continuous(breaks=seq(0, 300, 100))
CPPa <- ggplot(CPPagg, aes(x=t_ms, y=meanCPP, group = VQlab)) + geom_line(aes(color=VQlab, linetype=VQlab), size =1) + geom_ribbon(aes(ymin=meanCPP-sd, ymax=meanCPP+sd, fill=VQlab), alpha=0.2) + labs(x=element_blank(), y = "CPP (dB)") + scale_colour_discrete(name = "OQ", labels=c("0.4", "0.5", "0.6")) + scale_fill_discrete(name = "OQ", labels=c("0.4", "0.5", "0.6")) + scale_linetype_discrete(name = "OQ",, labels=c("0.4", "0.5", "0.6"))+ theme(text = element_text(size = 10), legend.position="top", legend.key.size=unit(0.5, "line"), legend.text = element_text(size=8), legend.title = element_text(size=8))+ scale_x_continuous(breaks=seq(0, 300, 100))


## load PraatSauce data for tuq
ps <- read.csv('Chrusynth_tuq.txt')

# Fill names of columns and turn characters into factors
names(ps)[3] <- 'F1lab'
names(ps)[4] <- 'f0lab'
names(ps)[5] <- 'VQlab'
ps$F1lab <- factor(ps$F1lab)
ps$f0lab <- factor(ps$f0lab)
ps$VQlab <- factor(ps$VQlab)

## Create charts

# Remove first two t points to remove effect of onset on measures
ps <- droplevels(subset(ps, t > 2 & t < 60))  
ps$t_ms <- ps$t * 5

f0agg <- ddply(ps, c("f0lab", "t_ms"), summarize, N = sum(!is.na(f0)), meanf0 = mean(f0, na.rm=TRUE), sd = sd(f0, na.rm=TRUE), se = sd / sqrt(N))
F1agg <- ddply(ps, c("F1lab", "t_ms"), summarize, N = sum(!is.na(F1)), meanF1 = mean(F1, na.rm=TRUE), sd = sd(F1, na.rm=TRUE), se = sd / sqrt(N))
H1H2agg <- ddply(ps, c("VQlab", "t_ms"), summarize, N = sum(!is.na(H1H2c)), meanH1H2c = mean(H1H2c, na.rm=TRUE), sd = sd(H1H2c, na.rm=TRUE), se = sd / sqrt(N))
CPPagg <- ddply(ps, c("VQlab", "t_ms"), summarize, N = sum(!is.na(CPP)), meanCPP = mean(CPP, na.rm=TRUE), sd = sd(CPP, na.rm=TRUE), se = sd / sqrt(N))

##charts for paper
F1u <- ggplot(F1agg, aes(x=t_ms, y=meanF1, group = F1lab)) + geom_line(aes(color=F1lab, linetype=F1lab), size =1) + geom_ribbon(aes(ymin=meanF1-sd, ymax=meanF1+sd, fill=F1lab), alpha=0.2) + labs(x=element_blank(), y = "F1 (Hz)") + scale_colour_discrete(name = "F1") + scale_fill_discrete(name = "F1") + scale_linetype_discrete(name = "F1") + theme(text = element_text(size = 10), legend.position="top", legend.key.size=unit(0.5, "line"), legend.text = element_text(size=8), legend.title = element_text(size=8)) + scale_x_continuous(breaks=seq(0, 300, 100)) 
f0u <- ggplot(f0agg, aes(x=t_ms, y=meanf0, group = f0lab)) + geom_line(aes(color=f0lab, linetype=f0lab), size =1) + geom_ribbon(aes(ymin=meanf0-sd, ymax=meanf0+sd, fill=f0lab), alpha=0.2) + labs(x =element_blank(), y = "f0 (Hz)") + scale_colour_discrete(name = "f0") + scale_fill_discrete(name = "f0") + scale_linetype_discrete(name = "f0") + theme(text = element_text(size = 10), legend.position="top", legend.key.size=unit(0.5, "line"), legend.text = element_text(size=8), legend.title = element_text(size=8)) + scale_x_continuous(breaks=seq(0, 300, 100))
H1H2u <- ggplot(H1H2agg, aes(x=t_ms, y=meanH1H2c, group = VQlab)) + geom_line(aes(color=VQlab, linetype=VQlab), size =1) + geom_ribbon(aes(ymin=meanH1H2c-sd, ymax=meanH1H2c+sd, fill=VQlab), alpha=0.2) + labs(x=element_blank(), y = "H1*-H2* (dB)") + scale_colour_discrete(name = "OQ", labels=c("0.4", "0.5", "0.6")) + scale_fill_discrete(name = "OQ", labels=c("0.4", "0.5", "0.6")) + scale_linetype_discrete(name = "OQ",, labels=c("0.4", "0.5", "0.6")) + theme(text = element_text(size = 10), legend.position="top", legend.key.size=unit(0.5, "line"), legend.text = element_text(size=8), legend.title = element_text(size=8)) + scale_x_continuous(breaks=seq(0, 300, 100))
CPPu <- ggplot(CPPagg, aes(x=t_ms, y=meanCPP, group = VQlab)) + geom_line(aes(color=VQlab, linetype=VQlab), size =1) + geom_ribbon(aes(ymin=meanCPP-sd, ymax=meanCPP+sd, fill=VQlab), alpha=0.2) + labs(x=element_blank(), y = "CPP (dB)") + scale_colour_discrete(name = "OQ", labels=c("0.4", "0.5", "0.6")) + scale_fill_discrete(name = "OQ", labels=c("0.4", "0.5", "0.6")) + scale_linetype_discrete(name = "OQ",, labels=c("0.4", "0.5", "0.6"))+ theme(text = element_text(size = 10), legend.position="top", legend.key.size=unit(0.5, "line"), legend.text = element_text(size=8), legend.title = element_text(size=8))+ scale_x_continuous(breaks=seq(0, 300, 100))


## Merge mta and tuq charts

figure <- ggarrange(f0a, H1H2a, CPPa, F1a, f0u, H1H2u, CPPu, F1u, ncol = 4, nrow = 2)
figure <- annotate_figure(figure, bottom = text_grob("Time from vowel onset (ms)", size=10))

bmp("Fig 13 mta_tuq_resynth.bmp", units="cm", width=16, height=10, res=600)
figure
dev.off()



